@@ -1,7 +1,7 @@
 /*
 
 Copyright (C) 2002 Hayato Fujiwara <h_fujiwara@users.sourceforge.net>
-Copyright (C) 2010-2015 Olaf Till <i7tiol@t-online.de>
+Copyright (C) 2010-2016 Olaf Till <i7tiol@t-online.de>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -23,14 +23,12 @@ along with this program; If not, see <http://www.gnu.org/licenses/>.
 
 #include <octave/oct.h>
 #include <octave/symtab.h>
-
-#include <oct-env.h>
-
-#include <file-io.h>
-#include <sighandlers.h>
-#include <parse.h>
-#include <cmd-edit.h>
-#include <toplev.h>
+#include <octave/oct-env.h>
+#include <octave/file-io.h>
+#include <octave/sighandlers.h>
+#include <octave/parse.h>
+#include <octave/cmd-edit.h>
+#include <octave/toplev.h>
 
 #include <sys/socket.h>
 #include <sys/types.h>
@@ -140,16 +138,16 @@ reval_loop (octave_parallel_stream &cstr)
           dsprintf ("reval loop, after successful network_recv_string\n");
         }
 
+      bool err;
       int p_err;
-      eval_string (s, false, p_err, 0);
+      SET_ERR (eval_string (s, false, p_err, 0), err);
       dsprintf ("reval loop, after evaluating string\n");
 
       uint32_t nl = 0;
-      if (error_state)
+      if (err)
         {
-          dsprintf ("reval loop, error state was set\n");
+          dsprintf ("reval loop, evaluation caused error\n");
           nl = 1;
-          error_state = 0;
         }
       else if (p_err)
         {
@@ -234,18 +232,65 @@ static int check_and_write_pidfile (const char *fname, int pid)
 
 DEFUN_DLD (pserver, args, ,
            "-*- texinfo -*-\n\
-@deftypefn {Loadable Function} pserver ()\n\
-@deftypefnx {Loadable Function} pserver (@var{options})\n\
+@deftypefn {Loadable Function} {} pserver ()\n\
+@deftypefnx {Loadable Function} {} pserver (@var{options})\n\
 This function starts a server of the parallel cluster and should be called once at any server machine.\n\
-It is important to call this function in a way assuring that Octave is quit as soon as the function returns, i.e. call it e.g. like @code{octave --eval \"pserver\"} or @code{octave --eval \"pserver (struct ('fieldname', value))\"}.\n\n\
-If a directory path corresponding to the current directory of the client exists on the server machine, it will be used as the servers current directory for the respective client (multiple clients are possible). Otherwise, @code{/tmp} will be used. The Octave functions the server is supposed to call and the files it possibly has to access must be available at the server machine. This can e.g. be achieved by having the server machine mount a network file system (which is outside the scope of this package documentation).\n\n\
-The parent server process can only be terminated by sending it a signal. The pid of this process, as long as it is running, will be stored in the file @code{/tmp/.octave-<hostname>.pid}.\n\n\
-If a connection is accepted from a client, the server collects a network identifier and the names of all server machines of the network from the client. Then, connections are automatically established between all machines of the network. Data exchange will be possible between all machines (client or server) in both directions. Commands can only be sent from the client to any server.\n\n\
-The opaque variable holding the network connections, in the same order as in the corresponding variable returned by @code{pconnect}, is accessible under the variable name @code{sockets} at the server side. Do not overwrite or clear this variable. The own server machine will also be contained at some index position of this variable, but will not correspond to a real connection. See @code{pconnect} for further information.\n\n\
-@var{options}: structure of options; field @code{use_tls} is @code{true} by default (TLS with SRP authentication); if set to @code{false}, there will be no encryption or authentication. Field @code{auth_file} can be set to an alternative path to the file with authentication information (see below).\n\n\
-The client and the server must both use or both not use TLS. If TLS is switched off, different measures must be taken to protect ports 12501 and 12502 at the servers and the client against unauthorized access; e.g. by a firewall or by physical isolation of the network.\n\n\
-For using TLS, authorization data must be present at the server machine. These data can conveniently be generated by @code{parallel_generate_srp_data}; the helptext of the latter function documents the expected location of these data.\n\n\
-The SRP password will be sent over the encrypted TLS channel from the client to each server, to avoid permanently storing passwords at the server for server-to-server data connections. Due to inevitable usage of external libraries, memory with sensitive data can, however, be on the swap device even after shutdown of the application.\n\n\
+\n\
+It is important to call this function in a way assuring that Octave is\n\
+quit as soon as the function returns, i.e. call it e.g. like\n\
+@code{octave --eval \"pserver\"} or @code{octave --eval \"pserver\n\
+(struct ('fieldname', value))\"}.\n\
+\n\
+If a directory path corresponding to the current directory of the\n\
+client exists on the server machine, it will be used as the servers\n\
+current directory for the respective client (multiple clients are\n\
+possible). Otherwise, @code{/tmp} will be used. The Octave functions\n\
+the server is supposed to call and the files it possibly has to access\n\
+must be available at the server machine. This can e.g. be achieved by\n\
+having the server machine mount a network file system (which is\n\
+outside the scope of this package documentation).\n\
+\n\
+The parent server process can only be terminated by sending it a\n\
+signal. The pid of this process, as long as it is running, will be\n\
+stored in the file @code{/tmp/.octave-<hostname>.pid}.\n\
+\n\
+If a connection is accepted from a client, the server collects a\n\
+network identifier and the names of all server machines of the network\n\
+from the client. Then, connections are automatically established\n\
+between all machines of the network. Data exchange will be possible\n\
+between all machines (client or server) in both directions. Commands\n\
+can only be sent from the client to any server.\n\
+\n\
+The opaque variable holding the network connections, in the same order\n\
+as in the corresponding variable returned by @code{pconnect}, is\n\
+accessible under the variable name @code{sockets} at the server\n\
+side. Do not overwrite or clear this variable. The own server machine\n\
+will also be contained at some index position of this variable, but\n\
+will not correspond to a real connection. See @code{pconnect} for\n\
+further information.\n\
+\n\
+@var{options}: structure of options; field @code{use_tls} is\n\
+@code{true} by default (TLS with SRP authentication); if set to\n\
+@code{false}, there will be no encryption or authentication. Field\n\
+@code{auth_file} can be set to an alternative path to the file with\n\
+authentication information (see below).\n\
+\n\
+The client and the server must both use or both not use TLS. If TLS is\n\
+switched off, different measures must be taken to protect ports 12501\n\
+and 12502 at the servers and the client against unauthorized access;\n\
+e.g. by a firewall or by physical isolation of the network.\n\
+\n\
+For using TLS, authorization data must be present at the server\n\
+machine. These data can conveniently be generated by\n\
+@code{parallel_generate_srp_data}; the helptext of the latter function\n\
+documents the expected location of these data.\n\
+\n\
+The SRP password will be sent over the encrypted TLS channel from the\n\
+client to each server, to avoid permanently storing passwords at the\n\
+server for server-to-server data connections. Due to inevitable usage\n\
+of external libraries, memory with sensitive data can, however, be on\n\
+the swap device even after shutdown of the application.\n\
+\n\
 @seealso{pconnect, reval, psend, precv, sclose, parallel_generate_srp_data, select_sockets}\n\
 @end deftypefn")
 {
@@ -305,15 +350,9 @@ The SRP password will be sent over the encrypted TLS channel from the client to
     {
       octave_scalar_map options;
 
-      options = args(0).scalar_map_value ();
-
-      if (error_state)
-        {
-          error ("%s: could not convert second argument to scalar structure",
-                 fname.c_str ());
-
-          return retval;
-        }
+      CHECK_ERROR (options = args(0).scalar_map_value (), retval,
+                   "%s: could not convert second argument to scalar structure",
+                   fname.c_str ());
 
       octave_value tmp;
 
@@ -322,15 +361,9 @@ The SRP password will be sent over the encrypted TLS channel from the client to
 
       if (tmp.is_defined ())
         {
-          use_gnutls = tmp.bool_value ();
-
-          if (error_state)
-            {
-              error ("%s: could not convert option 'use_tls' to bool",
-                     fname.c_str ());
-
-              return retval;
-            }
+          CHECK_ERROR (use_gnutls = tmp.bool_value (), retval,
+                       "%s: could not convert option 'use_tls' to bool",
+                       fname.c_str ());
         }
 
       // custom authentication file
@@ -338,15 +371,9 @@ The SRP password will be sent over the encrypted TLS channel from the client to
 
       if (tmp.is_defined ())
         {
-          cafile = tmp.string_value ();
-
-          if (error_state)
-            {
-              error ("%s: could not convert option 'auth_file' to string",
-                     fname.c_str ());
-
-              return retval;
-            }
+          CHECK_ERROR (cafile = tmp.string_value (), retval,
+                       "%s: could not convert option 'auth_file' to string",
+                       fname.c_str ());
         }
     } // args.length () == 1
 
@@ -587,20 +614,13 @@ The SRP password will be sent over the encrypted TLS channel from the client to
                     {
                       octave_value_list f_args (1);
                       f_args(0) = octave_value ("prefix");
-                      octave_value_list f_ret = feval ("pkg", f_args, 1);
-                      if (error_state)
-                        {
-                          error ("%s: could not get prefix from pkg",
-                                 fname.c_str ());
-                          _exit (1);
-                        }
-                      cafile = f_ret(0).string_value ();
-                      if (error_state)
-                        {
-                          error ("%s: could not convert output of pkg ('prefix') to string)",
-                                 fname.c_str ());
-                          _exit (1);
-                        }
+                      octave_value_list f_ret;
+                      CHECK_ERROR_EXIT1 (f_ret = feval ("pkg", f_args, 1),
+                                         "%s: could not get prefix from pkg",
+                                         fname.c_str ());
+                      CHECK_ERROR_EXIT1 (cafile = f_ret(0).string_value (),
+                                         "%s: could not convert output of pkg ('prefix') to string)",
+                                         fname.c_str ());
                       cafile = cafile + file_ops::dir_sep_str () +
                         "parallel-srp-data" + file_ops::dir_sep_str () +
                         file_ops::dir_sep_str () + "server" +
@@ -887,10 +907,10 @@ The SRP password will be sent over the encrypted TLS channel from the client to
                   else
                     {
                       conn->get_data_stream ()->network_send_4byteint (0);
-                      conn->connection_read_header ();
+                      int err = conn->connection_read_header ();
                       write_header (conn->get_data_stream ()->get_ostream (),
                                     LS_BINARY);
-                      if (error_state || ! conn->get_data_stream ()->good ())
+                      if (err || ! conn->get_data_stream ()->good ())
                         {
                           error ("communication error in initialization");
                           _exit (1);
@@ -953,7 +973,7 @@ The SRP password will be sent over the encrypted TLS channel from the client to
               if (getaddrinfo (hosts(i - 1).c_str (), "12501", &hints, &ai))
                 {
                   error ("getaddrinfo returned an error");
-                  return retval;
+                  _exit (1);
                 }
               struct __aiguard
               {
@@ -1048,8 +1068,7 @@ The SRP password will be sent over the encrypted TLS channel from the client to
                           write_header
                             (conn->get_data_stream ()->get_ostream (),
                              LS_BINARY);
-                          conn->connection_read_header ();
-                          if (error_state ||
+                          if (conn->connection_read_header () ||
                               ! conn->get_data_stream ()->good ())
                             {
                               error ("communication error in initialization");
